@@ -1,5 +1,5 @@
 
-/*  $Header: /cvs/gnome/gcalctool/gcalctool/calctool.c,v 1.59 2003/11/24 21:00:20 richb Exp $
+/*  $Header: /cvs/gnome/gcalctool/gcalctool/calctool.c,v 1.64 2003/12/23 14:21:12 richb Exp $
  *
  *  Copyright (c) 1987-2003 Sun Microsystems, Inc. All Rights Reserved.
  *           
@@ -60,7 +60,7 @@ char *base_desc[]  = {         /* Tooltips for each base value. */
 
 char *calc_res[] = {
     "accuracy", "base", "display", "mode", "showregisters", "trigtype",
-    "removezeroes"
+    "showzeroes", "showthousands"
 };
 
 char *dtype_str[] = {          /* Strings for each display mode value. */
@@ -91,7 +91,7 @@ char *ttype_str[] = {          /* Strings for each trig type value. */
 
 char *ttype_desc[] = {         /* Tooltips for each trig type value. */
     N_("Set trigonometric type to degrees"), 
-    N_("Set trigonometric type to gradients"), 
+    N_("Set trigonometric type to gradians"), 
     N_("Set trigonometric type to radians")
 };
 
@@ -171,8 +171,8 @@ struct button b_buttons[B_NOBUTTONS] = {   /* Basic mode button values. */
     N_("/"),
     N_("Divide"),
     NULL,
-    { 0,         0,             0,      0 },
-    { GDK_slash, GDK_KP_Divide, GDK_R5, 0 },
+    { 0,         0,             0,      GDK_SHIFT_MASK, 0 },
+    { GDK_slash, GDK_KP_Divide, GDK_R5, GDK_slash,      0 },
     '/',
     M_NONE,
     do_calc
@@ -408,7 +408,7 @@ struct button b_buttons[B_NOBUTTONS] = {   /* Basic mode button values. */
     N_("Calculate result"),
     NULL,
     { 0,         0,            0,          GDK_SHIFT_MASK, 0 },
-    { GDK_equal, GDK_KP_Enter, GDK_Return, GDK_Return,     0 },
+    { GDK_equal, GDK_KP_Enter, GDK_Return, GDK_equal,      0 },
     '=',
     M_NONE,
     do_calc
@@ -967,8 +967,8 @@ struct button s_buttons[S_NOBUTTONS] = {   /* Scientific mode button values. */
     N_("Xor"),
     N_("Logical XOR"),
     NULL,
-    { GDK_SHIFT_MASK, 0 },
-    { GDK_caret, 0 },
+    { GDK_SHIFT_MASK, GDK_SHIFT_MASK,  0 },
+    { GDK_caret,      GDK_asciicircum, 0 },
     '^',
     M_NONE,
     do_calc
@@ -1017,7 +1017,8 @@ do_calctool(int argc, char **argv)
         }
     }
 
-    v->radix_char = get_radix_char();   /* Locale specific radix character. */
+    v->radix = get_radix();    /* Locale specific radix string. */
+    v->tsep  = get_tsep();     /* Locale specific thousands seperator. */
 
     init_text();               /* Setup text strings depending upon language. */
     init_vars();               /* Setup default values for variables. */
